package dev.flyfish.framework.compiler.core;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.util.StreamUtils;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Load class from byte[] which is compiled in memory.
 *
 * @author michael
 */
class MemoryClassLoader extends DynamicClassLoader {

    // class name to class bytes:
    private final Map<String, byte[]> classBytes = new ConcurrentHashMap<>();

    @Override
    protected Class<?> findClass(String name) throws ClassNotFoundException {
        byte[] buf = classBytes.get(name);
        if (buf == null) {
            return super.findClass(name);
        }
        classBytes.remove(name);
        return defineClass(name, buf);
    }

    @Override
    public Map<String, byte[]> getClassBytes() {
        return classBytes;
    }

    @Override
    public void replace(String name, String path) {
        String actualPath = "classpath:" + path;
        ResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
        try {
            Resource[] resources = resolver.getResources(actualPath);
            if (ArrayUtils.isNotEmpty(resources)) {
                byte[] content = StreamUtils.copyToByteArray(resources[0].getInputStream());
                classBytes.put(name, content);
                defineClass(name, content);
            }
        } catch (IOException ignored) {
        }
    }

    /**
     * 修复获取package为null的问题，定义并验证class
     *
     * @param name 名称
     * @param buf  二进制数据
     * @return 结果
     */
    @SuppressWarnings("all")
    private Class<?> defineClass(String name, byte[] buf) {
        // 先定义class，然后在上下文基础上判断package有效性
        Class<?> defined = defineClass(name, buf, 0, buf.length);
        synchronized (defined) {
            // 补偿定义包，防止缺失
            if (null == defined.getPackage()) {
                // 包名
                String packageName = StringUtils.substringBeforeLast(name, ".");
                definePackage(packageName, null, null,
                        null, null, null, null, null);
            }
        }
        return defined;
    }
}
