package dev.flyfish.framework.query;

import java.util.Collection;
import java.util.List;

/**
 * 查询条件
 *
 * @param <C> 条件本类型
 * @author wangyu
 */
public interface QueryCondition<C extends QueryChain<C, ?>> {

    /**
     * 相等判定
     *
     * @param value 值
     * @return 结果
     */
    C eq(Object value);

    /**
     * 判定某列的值中存在指定值，特指json array数据类型且子类型中带有id的场景
     * 用于兼容mongodb查询，mysql查询使用JSON_CONTAINS进行判定。
     * 当且仅当mongodb会拼接.$id，mysql一律匹配id字段
     * <p>
     * 等价于 .eq(value)
     *
     * @param value 值
     * @return 结果
     */
    C hasId(Object value);

    /**
     * 对于mongodb，自动处理。对于关系型数据库，代表json array中是否包含对应值，值仅支持基本数据类型
     * <p>
     * 等价于 .eq(value)
     *
     * @param value 基本数据类型的值
     * @return 结果
     */
    C has(Object value);

    /**
     * 不等判定
     *
     * @param value 值
     * @return 结果
     */
    C ne(Object value);

    /**
     * 值介于两者之间
     *
     * @param items 双值列表
     * @return 结果
     */
    C between(List<?> items);

    /**
     * 模糊匹配，这里是全模糊
     *
     * @param keyword 查询关键字
     * @return 结果
     */
    C like(String keyword);

    /**
     * 根据指定的方向进行模糊查询
     *
     * @param keyword   关键字
     * @param direction 方向，可以匹配开头和结尾
     * @return 结果
     */
    C like(String keyword, Queries.Direction direction);

    /**
     * 判定为空
     *
     * @return 结果
     */
    C isNull();

    /**
     * 包含在内
     *
     * @param values 集合
     * @return 结果
     */
    C in(Collection<?> values);

    /**
     * 包含在内
     * 注意，需要根据字段类型推断。
     * 如果是json数组，需要进行双向匹配
     *
     * @param values 值们
     * @return 结果
     */
    C in(Object... values);

    /**
     * 不包含在内
     * 注意，需要根据字段类型推断。
     * 如果是json数组，需要进行双向匹配
     *
     * @param values 值们
     * @return 结果
     */
    C nin(Object... values);

    /**
     * 不包含在内
     *
     * @param values 集合
     * @return 结果
     */
    C nin(Collection<?> values);

    /**
     * 小于指定的值
     *
     * @param value 类型
     * @return 结果
     */
    C lt(Object value);

    /**
     * 小于等于指定的值
     *
     * @param value 类型
     * @return 结果
     */
    C lte(Object value);

    /**
     * 大于指定的值
     *
     * @param value 类型
     * @return 结果
     */
    C gt(Object value);

    /**
     * 大于等于指定的值
     *
     * @param value 类型
     * @return 结果
     */
    C gte(Object value);

}
