package dev.flyfish.framework.query.chain;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.query.*;
import dev.flyfish.framework.query.*;
import dev.flyfish.framework.query.support.DomainFunction;
import lombok.RequiredArgsConstructor;

import java.util.Collections;

/**
 * 默认的查询修改逻辑
 *
 * @author wangyu
 * <p>
 * 不修改原条件，而是在此基础上新增
 * 理论上查询不允许直接修改，而是通过该类做一个桥接
 * 也可以修改链的形式
 */
@RequiredArgsConstructor
public class DefaultQueryMutation implements QueryMutation {

    private final QueryChain<?, ?> previous;

    private final boolean within;

    /**
     * 根据实际的参数合并查询链
     *
     * @param newChain 新链
     * @param <P>      参数类型
     * @param <C>      查询链类型
     * @return 结果
     */
    private <P, C extends QueryChain<C, P>> C merge(C newChain) {
        if (within) {
            // 内联模式，旧条件不修改
            newChain.and(Collections.singletonList(previous));
        } else {
            // 修改模式，直接修改
            previous.and(newChain);
        }
        return newChain;
    }

    /**
     * 以lambda的形式修改
     * 注意，如果之前用过lambda的形式，此处应该校验类型
     *
     * @param getter 方法引用
     * @return 结果
     */
    @Override
    public <T extends Domain> QueryCondition<LambdaQueryChain<T>> and(DomainFunction<T> getter) {
        return merge(new DefaultLambdaQueryChain<T>()).and(getter);
    }

    /**
     * 以column的形式修改
     *
     * @param column 列名
     * @return 结果
     */
    @Override
    public QueryCondition<NamedQueryChain> and(String column) {
        return merge(new DefaultNamedQueryChain()).and(column);
    }

    /**
     * 直接以and拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    @Override
    public <T extends Domain> LambdaQueryChain<T> and(LambdaQueryChain<T> chain) {
        return merge(chain);
    }

    /**
     * 直接以and拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    @Override
    public NamedQueryChain and(NamedQueryChain chain) {
        return merge(chain);
    }

    /**
     * 以lambda的形式修改
     * 注意，如果之前用过lambda的形式，此处应该校验类型
     *
     * @param getter 方法引用
     * @return 结果
     */
    @Override
    public <T extends Domain> QueryCondition<LambdaQueryChain<T>> or(DomainFunction<T> getter) {
        return merge(new DefaultLambdaQueryChain<T>()).or(getter);
    }

    /**
     * 以column的形式修改
     *
     * @param column 列名
     * @return 结果
     */
    @Override
    public QueryCondition<NamedQueryChain> or(String column) {
        return merge(new DefaultNamedQueryChain()).or(column);
    }

    /**
     * 直接以or拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    @Override
    public <T extends Domain> LambdaQueryChain<T> or(LambdaQueryChain<T> chain) {
        return merge(chain);
    }

    /**
     * 直接以or拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    @Override
    public NamedQueryChain or(NamedQueryChain chain) {
        return merge(chain);
    }
}
