package dev.flyfish.framework.query.spi.adaptor;

import dev.flyfish.framework.query.Queries;

import java.util.Collection;
import java.util.List;

/**
 * 查询适配器
 * 直接根据条件操作得到查询条件片段
 * 适配不同的数据库抽取通用操作
 *
 * @author wangyu
 * 由工厂进行实例化，并输出
 */
public interface CriteriaAdaptor<C> {

    /**
     * 判断某个查询是否为空
     *
     * @param criteria 查询
     * @return 结果
     */
    boolean isEmpty(C criteria);

    /**
     * 一个空的查询，用于起手
     *
     * @return 结果
     */
    C empty();

    /**
     * 且的关系进行拼接
     *
     * @param first 起始条件
     * @param other 其他条件
     * @return 结果
     */
    C and(C first, C other);

    /**
     * 或的关系进行拼接
     *
     * @param first 起始条件
     * @param other 其他条件
     * @return 结果
     */
    C or(C first, C other);

    /**
     * 相等判定
     *
     * @param column 列名
     * @param value  值
     * @return 结果
     */
    C eq(String column, Object value);

    /**
     * 判定某列的值中存在指定值，特指json array数据类型且子类型中带有id的场景
     * 用于兼容mongodb查询，mysql查询使用JSON_CONTAINS进行判定。
     * 当且仅当mongodb会拼接.$id，mysql一律匹配id字段
     * <p>
     * 等价于 .eq(value)
     *
     * @param column 列名
     * @param value  值
     * @return 结果
     */
    C hasId(String column, Object value);

    /**
     * 对于mongodb，自动处理。对于关系型数据库，代表json array中是否包含对应值，值仅支持基本数据类型
     * <p>
     * 等价于 .eq(value)
     *
     * @param column 列名
     * @param value  基本数据类型的值
     * @return 结果
     */
    C has(String column, Object value);

    /**
     * 不等判定
     *
     * @param column 列名
     * @param value  值
     * @return 结果
     */
    C ne(String column, Object value);

    /**
     * 值介于两者之间
     *
     * @param column 列名
     * @param items  双值列表
     * @return 结果
     */
    C between(String column, List<?> items);

    /**
     * 根据指定的方向进行模糊查询
     *
     * @param column    列名
     * @param keyword   关键字
     * @param direction 方向，可以匹配开头和结尾
     * @return 结果
     */
    C like(String column, String keyword, Queries.Direction direction);

    /**
     * 判定为空
     *
     * @param column 列名
     * @return 结果
     */
    C isNull(String column);

    /**
     * 小于指定的值
     *
     * @param column 列名
     * @param value  类型
     * @return 结果
     */
    C lt(String column, Object value);

    /**
     * 小于等于指定的值
     *
     * @param column 列名
     * @param value  类型
     * @return 结果
     */
    C lte(String column, Object value);

    /**
     * 大于指定的值
     *
     * @param column 列名
     * @param value  类型
     * @return 结果
     */
    C gt(String column, Object value);

    /**
     * 大于等于指定的值
     *
     * @param column 列名
     * @param value  类型
     * @return 结果
     */
    C gte(String column, Object value);

    /**
     * 包含在内
     *
     * @param column 列名
     * @param values 集合
     * @return 结果
     */
    C in(String column, Collection<?> values);

    /**
     * 包含在内
     * 注意，需要根据字段类型推断。
     * 如果是json数组，需要进行双向匹配
     *
     * @param column 列名
     * @param values 值们
     * @return 结果
     */
    C in(String column, Object... values);

    /**
     * 不包含在内
     * 注意，需要根据字段类型推断。
     * 如果是json数组，需要进行双向匹配
     *
     * @param column 列名
     * @param values 值们
     * @return 结果
     */
    C nin(String column, Object... values);

    /**
     * 不包含在内
     *
     * @param column 列名
     * @param values 集合
     * @return 结果
     */
    C nin(String column, Collection<?> values);
}
