package dev.flyfish.framework.utils;

import lombok.RequiredArgsConstructor;
import org.springframework.data.redis.core.ReactiveRedisTemplate;
import org.springframework.data.redis.core.ReactiveStringRedisTemplate;
import org.springframework.data.util.CastUtils;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.Duration;
import java.time.temporal.ChronoUnit;
import java.util.List;
import java.util.Map;

/**
 * redis 工具类
 *
 * @author wangyu
 */
@RequiredArgsConstructor
public class ReactiveRedisOperations {

    private final ReactiveRedisTemplate<String, Object> redisTemplate;

    private final ReactiveStringRedisTemplate stringRedisTemplate;

    /**
     * 指定缓存失效时间
     *
     * @param key  键
     * @param time 时间(秒)
     * @return 是否成功指定
     */
    public Mono<Boolean> expire(String key, long time) {
        return redisTemplate.expire(key, Duration.of(time, ChronoUnit.SECONDS))
                .doOnError(Throwable::printStackTrace)
                .onErrorReturn(false);
    }

    /**
     * 根据key 获取过期时间
     *
     * @param key 键 不能为null
     * @return 时间(秒) 返回0代表为永久有效
     */
    public Mono<Long> getExpire(String key) {
        return redisTemplate.getExpire(key).map(Duration::getSeconds);
    }

    /**
     * 判断key是否存在
     *
     * @param key 键
     * @return true 存在 false不存在
     */
    public Mono<Boolean> hasKey(String key) {
        return redisTemplate.hasKey(key).doOnError(Throwable::printStackTrace).onErrorReturn(false);
    }

    /**
     * 删除缓存
     *
     * @param key 可以传一个值 或多个
     */
    public Mono<Long> del(String... key) {
        if (key != null && key.length > 0) {
            if (key.length == 1) {
                return redisTemplate.delete(key[0]);
            } else {
                return redisTemplate.delete(key);
            }
        }
        return Mono.just(0L);
    }

    /**
     * 删除缓存
     *
     * @param key 可以传一个值 或多个
     */
    public Mono<Long> del(Flux<String> key) {
        return redisTemplate.delete(key);
    }


    // ============================String=============================

    /**
     * 普通缓存获取
     *
     * @param key 键
     * @return 值
     */
    public <T> Mono<T> get(String key) {
        if (key == null) {
            return null;
        }
        try {
            return redisTemplate.opsForValue().get(key).map(CastUtils::cast);
        } catch (Exception e) {
            return stringRedisTemplate.opsForValue().get(key).map(CastUtils::cast);
        }
    }

    /**
     * 获取string
     *
     * @param key 键
     * @return 结果
     */
    public Mono<String> getString(String key) {
        return null == key ? Mono.empty() : stringRedisTemplate.opsForValue().get(key);
    }

    /**
     * 获取多个key值
     *
     * @param key 键
     * @return 结果
     */
    public Flux<String> getKeys(String key) {
        return null == key ? Flux.empty() : redisTemplate.keys(key);
    }

    /**
     * 普通缓存放入
     *
     * @param key   键
     * @param value 值
     * @return true成功 false失败
     */
    public Mono<Boolean> set(String key, Object value) {
        return redisTemplate.opsForValue().set(key, value)
                .doOnError(Throwable::printStackTrace).onErrorReturn(false);
    }

    public Mono<Boolean> set(String key, String value) {
        return stringRedisTemplate.opsForValue().set(key, value)
                .doOnError(Throwable::printStackTrace).onErrorReturn(false);
    }

    /**
     * 普通缓存放入并设置时间
     *
     * @param key   键
     * @param value 值
     * @param time  时间(秒) time要大于0 如果time小于等于0 将设置无限期
     * @return true成功 false 失败
     */
    public Mono<Boolean> set(String key, Object value, long time) {
        if (time > 0) {
            return redisTemplate.opsForValue().set(key, value, Duration.ofSeconds(time))
                    .doOnError(Throwable::printStackTrace).onErrorReturn(false);
        } else {
            return set(key, value);
        }
    }

    /**
     * 普通缓存放入并设置时间
     *
     * @param key      键
     * @param value    值
     * @param duration 时间(秒) time要大于0 如果time小于等于0 将设置无限期
     * @return true成功 false 失败
     */
    public Mono<Boolean> set(String key, Object value, Duration duration) {
        if (duration.getSeconds() > 0) {
            return redisTemplate.opsForValue().set(key, value, duration)
                    .doOnError(Throwable::printStackTrace).onErrorReturn(false);
        } else {
            return set(key, value);
        }
    }

    /**
     * 递增
     *
     * @param key   键
     * @param delta 要增加几(大于0)
     * @return 结果
     */
    public Mono<Long> incr(String key, long delta) {
        if (delta < 0) {
            return Mono.error(new RuntimeException("递增因子必须大于0"));
        }
        return redisTemplate.opsForValue().increment(key, delta);
    }

    /**
     * 递减
     *
     * @param key   键
     * @param delta 要减少几(小于0)
     * @return
     */
    public Mono<Long> decr(String key, long delta) {
        if (delta < 0) {
            return Mono.error(new RuntimeException("递减因子必须大于0"));
        }
        return redisTemplate.opsForValue().increment(key, -delta);
    }

    // ================================Map=================================

    /**
     * HashGet
     *
     * @param key  键 不能为null
     * @param item 项 不能为null
     * @return 值
     */
    public Mono<Object> hget(String key, String item) {
        return redisTemplate.opsForHash().get(key, item);
    }

    /**
     * 获取hashKey对应的所有键值
     *
     * @param key 键
     * @return 对应的多个键值
     */
    public Flux<Map.Entry<Object, Object>> hmget(String key) {
        return redisTemplate.opsForHash().entries(key);
    }

    /**
     * HashSet
     *
     * @param key 键
     * @param map 对应多个键值
     * @return true 成功 false 失败
     */
    public Mono<Boolean> hmset(String key, Map<String, Object> map) {
        return redisTemplate.opsForHash().putAll(key, map).doOnError(Throwable::printStackTrace).onErrorReturn(false);
    }

    /**
     * HashSet 并设置时间
     *
     * @param key  键
     * @param map  对应多个键值
     * @param time 时间(秒)
     * @return true成功 false失败
     */
    public Mono<Boolean> hmset(String key, Map<String, Object> map, long time) {
        return redisTemplate.opsForHash().putAll(key, map).doOnError(Throwable::printStackTrace)
                .flatMap(e -> time > 0 ? expire(key, time).thenReturn(true) : Mono.just(true))
                .onErrorReturn(false);
    }

    /**
     * 向一张hash表中放入数据,如果不存在将创建
     *
     * @param key   键
     * @param item  项
     * @param value 值
     * @return true 成功 false失败
     */
    public Mono<Boolean> hset(String key, String item, Object value) {
        return redisTemplate.opsForHash().put(key, item, value).doOnError(Throwable::printStackTrace).onErrorReturn(false);
    }

    /**
     * 向一张hash表中放入数据,如果不存在将创建
     *
     * @param key   键
     * @param item  项
     * @param value 值
     * @param time  时间(秒) 注意:如果已存在的hash表有时间,这里将会替换原有的时间
     * @return true 成功 false失败
     */
    public Mono<Boolean> hset(String key, String item, Object value, long time) {
        return redisTemplate.opsForHash().put(key, item, value)
                .doOnError(Throwable::printStackTrace)
                .flatMap(e -> time > 0 ? expire(key, time).thenReturn(true) : Mono.just(true))
                .onErrorReturn(false);
    }

    /**
     * 删除hash表中的值
     *
     * @param key 键 不能为null
     */
    public Mono<Boolean> hdel(String key) {
        return redisTemplate.opsForHash().delete(key);
    }

    /**
     * 判断hash表中是否有该项的值
     *
     * @param key  键 不能为null
     * @param item 项 不能为null
     * @return true 存在 false不存在
     */
    public Mono<Boolean> hHasKey(String key, String item) {
        return redisTemplate.opsForHash().hasKey(key, item);
    }

    /**
     * hash递增 如果不存在,就会创建一个 并把新增后的值返回
     *
     * @param key  键
     * @param item 项
     * @param by   要增加几(大于0)
     * @return
     */
    public Mono<Double> hincr(String key, String item, double by) {
        return redisTemplate.opsForHash().increment(key, item, by);
    }

    /**
     * hash递减
     *
     * @param key  键
     * @param item 项
     * @param by   要减少记(小于0)
     * @return
     */
    public Mono<Double> hdecr(String key, String item, double by) {
        return redisTemplate.opsForHash().increment(key, item, -by);
    }

    // ============================set=============================

    /**
     * 根据key获取Set中的所有值
     *
     * @param key 键
     * @return 所有值
     */
    public Flux<Object> sGet(String key) {
        return redisTemplate.opsForSet().members(key).doOnError(Throwable::printStackTrace)
                .onErrorResume(e -> Flux.empty());
    }

    /**
     * 根据value从一个set中查询,是否存在
     *
     * @param key   键
     * @param value 值
     * @return true 存在 false不存在
     */
    public Mono<Boolean> sHasKey(String key, Object value) {
        return redisTemplate.opsForSet().isMember(key, value).doOnError(Throwable::printStackTrace).onErrorReturn(false);
    }

    /**
     * 将数据放入set缓存
     *
     * @param key    键
     * @param values 值 可以是多个
     * @return 成功个数
     */
    public Mono<Long> sSet(String key, Object... values) {
        return redisTemplate.opsForSet().add(key, values).doOnError(Throwable::printStackTrace).onErrorReturn(0L);
    }

    /**
     * 将set数据放入缓存
     *
     * @param key    键
     * @param time   时间(秒)
     * @param values 值 可以是多个
     * @return 成功个数
     */
    public Mono<Long> sSetAndTime(String key, long time, Object... values) {
        return redisTemplate.opsForSet().add(key, values).doOnError(Throwable::printStackTrace)
                .flatMap(count -> time > 0 ? expire(key, time).thenReturn(count) : Mono.just(count))
                .onErrorReturn(0L);
    }

    /**
     * 获取set缓存的长度
     *
     * @param key 键
     * @return
     */
    public Mono<Long> sGetSetSize(String key) {
        return redisTemplate.opsForSet().size(key).doOnError(Throwable::printStackTrace).onErrorReturn(0L);
    }

    /**
     * 移除值为value的
     *
     * @param key    键
     * @param values 值 可以是多个
     * @return 移除的个数
     */
    public Mono<Long> setRemove(String key, Object... values) {
        return redisTemplate.opsForSet().remove(key, values).doOnError(Throwable::printStackTrace).onErrorReturn(0L);
    }
    // ===============================list=================================

    /**
     * 获取list缓存的内容
     *
     * @param key   键
     * @param start 开始
     * @param end   结束 0 到 -1代表所有值
     * @return
     */
    public Flux<Object> lGet(String key, long start, long end) {
        return redisTemplate.opsForList().range(key, start, end).doOnError(Throwable::printStackTrace)
                .onErrorResume(e -> Flux.empty());
    }

    /**
     * 获取list缓存的长度
     *
     * @param key 键
     * @return
     */
    public Mono<Long> lGetListSize(String key) {
        return redisTemplate.opsForList().size(key).doOnError(Throwable::printStackTrace).onErrorReturn(0L);
    }

    /**
     * 通过索引 获取list中的值
     *
     * @param key   键
     * @param index 索引 index>=0时， 0 表头，1 第二个元素，依次类推；index<0时，-1，表尾，-2倒数第二个元素，依次类推
     * @return
     */
    public Mono<Object> lGetIndex(String key, long index) {
        return redisTemplate.opsForList().index(key, index).doOnError(Throwable::printStackTrace)
                .onErrorResume(e -> Mono.empty());
    }

    /**
     * 将list放入缓存
     *
     * @param key   键
     * @param value 值
     * @return
     */
    public Mono<Boolean> lSet(String key, Object value) {
        return redisTemplate.opsForList().rightPush(key, value).doOnError(Throwable::printStackTrace)
                .map(r -> true)
                .onErrorReturn(false);
    }

    /**
     * 将list放入缓存
     *
     * @param key   键
     * @param value 值
     * @param time  时间(秒)
     * @return
     */
    public Mono<Boolean> lSet(String key, Object value, long time) {
        return redisTemplate.opsForList().rightPush(key, value)
                .doOnError(Throwable::printStackTrace)
                .flatMap(e -> time > 0 ? expire(key, time).thenReturn(true) : Mono.just(true))
                .onErrorReturn(false);
    }

    /**
     * 将list放入缓存
     *
     * @param key   键
     * @param value 值
     * @return
     */
    public Mono<Boolean> lSet(String key, List<Object> value) {
        return redisTemplate.opsForList().rightPushAll(key, value)
                .thenReturn(true)
                .doOnError(Throwable::printStackTrace)
                .onErrorReturn(false);
    }

    /**
     * 将list放入缓存
     *
     * @param key   键
     * @param value 值
     * @param time  时间(秒)
     * @return
     */
    public Mono<Boolean> lSet(String key, List<Object> value, long time) {
        return redisTemplate.opsForList().rightPushAll(key, value)
                .doOnError(Throwable::printStackTrace)
                .flatMap(e -> time > 0 ? expire(key, time).thenReturn(true) : Mono.just(true))
                .onErrorReturn(false);
    }

    /**
     * 根据索引修改list中的某条数据
     *
     * @param key   键
     * @param index 索引
     * @param value 值
     * @return
     */
    public Mono<Boolean> lUpdateIndex(String key, long index, Object value) {
        return redisTemplate.opsForList().set(key, index, value).doOnError(Throwable::printStackTrace)
                .onErrorReturn(false);
    }

    /**
     * 移除N个值为value
     *
     * @param key   键
     * @param count 移除多少个
     * @param value 值
     * @return 移除的个数
     */
    public Mono<Long> lRemove(String key, long count, Object value) {
        return redisTemplate.opsForList().remove(key, count, value).doOnError(Throwable::printStackTrace)
                .onErrorReturn(0L);
    }
}
